#include ".\MP3File.h"

CMP3File::CMP3File(void)
{
	p_szFName=NULL;
	p_bIsOpened=false;
	f_out=NULL;
	pMP3Buff=NULL;
	pTmpBuff=NULL;
	iTmpBuffPos=0;
	dwDataSize=0;
	// Paramtres d'encodage par dfaut
	p_dwBitrate=192;
	p_dwMaxBitrate=256;
	p_dwSampleRate=44100;
	p_wBits=16;
	p_nMode=BE_MP3_MODE_STEREO;
	p_bEnableVBR=false;
	// Obtention de la version de la Dll et du moteur d'encodage
	BE_VERSION	Version;
	beVersion(&Version);
	p_fDllVersion=(float)Version.byDLLMajorVersion;
	p_fDllVersion+=(float)(Version.byDLLMinorVersion/100.f);
	p_fEngineVersion=(float)Version.byMajorVersion;
	p_fEngineVersion+=(float)(Version.byMinorVersion/100.f);
}

CMP3File::~CMP3File(void)
{
	if (p_szFName!=NULL) delete[] p_szFName;
	if (pMP3Buff!=NULL) delete[] pMP3Buff;
	if (pTmpBuff!=NULL) delete[] pTmpBuff;
	if (p_bIsOpened) Close();
}

int CMP3File::SetFileName(char *szFName)
{
	if (p_bIsOpened==true) return 0;
	if (szFName==NULL) return 0;
	int i=(int)strlen(szFName);
	if (i==0) return 0;
	if (p_szFName!=NULL) delete[] p_szFName;
	p_szFName=new char[i+1];
	strcpy(p_szFName,szFName);
	return 1;
}

int CMP3File::Create()
{
	if (p_bIsOpened) return 0;
	if (p_szFName==NULL) return 0;
	f_out=fopen(p_szFName,"w+b");
	if (f_out==NULL) return 0;
	memset(&beConf,0,sizeof(BE_CONFIG));
	beConf.dwConfig = BE_CONFIG_LAME;

	beConf.format.LHV1.dwStructVersion= 1;
	beConf.format.LHV1.dwStructSize= sizeof(BE_CONFIG);		
	beConf.format.LHV1.dwSampleRate= p_dwSampleRate; // INPUT FREQUENCY
	beConf.format.LHV1.dwReSampleRate=0;// DON"T RESAMPLE
	beConf.format.LHV1.nMode=p_nMode;// OUTPUT IN MONO/STREO
	beConf.format.LHV1.dwBitrate=p_dwBitrate; // MINIMUM BIT RATE
	beConf.format.LHV1.nPreset=(p_bEnableVBR==true)?LQP_R3MIX:LQP_CBR; // QUALITY PRESET SETTING
	beConf.format.LHV1.dwMpegVersion=MPEG1; // MPEG VERSION (I or II)
	beConf.format.LHV1.dwPsyModel=0; // USE DEFAULT PSYCHOACOUSTIC MODEL 
	beConf.format.LHV1.dwEmphasis=0; // NO EMPHASIS TURNED ON
	beConf.format.LHV1.bOriginal=TRUE; // SET ORIGINAL FLAG
	beConf.format.LHV1.bWriteVBRHeader=TRUE; // Write INFO tag

	beConf.format.LHV1.dwMaxBitrate		= p_dwMaxBitrate;					// MAXIMUM BIT RATE
	beConf.format.LHV1.bEnableVBR			= p_bEnableVBR;					// USE VBR
	beConf.format.LHV1.bNoRes=TRUE; // No Bit resorvoir

	// Initialisation du stream MP3
	if (beInitStream(&beConf,&dwSmpls,&dwMP3Buff,&hbeStrm)!=BE_ERR_SUCCESSFUL)
	{
		MessageBox(GetActiveWindow(),"beInitStream failed...","Error",MB_ICONEXCLAMATION);
		fclose(f_out);
		return 0;
	}
	// Cration du buffer de retour
	if (pMP3Buff!=NULL) delete[] pMP3Buff;
	pMP3Buff = new BYTE[dwMP3Buff];
	// Cration du buffer temporaire
	if (pTmpBuff!=NULL) delete[] pTmpBuff;
	pTmpBuff = new BYTE[dwSmpls*8];
	// Vrification de la cration des buffers
	if((!pMP3Buff)||(!pTmpBuff))
	{
		beDeinitStream(hbeStrm,pMP3Buff,&dwNRes);
		fclose(f_out);
		MessageBox(GetActiveWindow(),"Out of memory ...","Error",MB_ICONERROR);
		return 0;
	}
	iTmpBuffPos=0;
	dwDataSize=0;
	p_bIsOpened=true;
	return 1;
}

int CMP3File::AddDatas(void *TheDatas,int DataSize)
{
	if ((TheDatas==NULL)||(DataSize==0)) return 0;
	if (p_bIsOpened==false) return 0;
	DWORD dwCoded,iPqSize=(dwSmpls*sizeof(SHORT));
	int i,iDataPos;
	char *pSrcDatas=(char*)TheDatas;
	// Si pas assez de donnes  encoder, alors on stocke pour ventuellement
	// le faire la prochaine fois
	if ((iTmpBuffPos+DataSize)<(int)iPqSize)
	{
		memcpy(pTmpBuff+iTmpBuffPos,TheDatas,DataSize);
		iTmpBuffPos+=DataSize;
		return 1;
	}
	iDataPos=0;
	while((int)(iDataPos+iPqSize)<(int)DataSize)
	{
		// On copie la quantit de donnes ncessaire  l'encodage dans le buffer temporaire
		i=iPqSize-iTmpBuffPos;
		memcpy(pTmpBuff+iTmpBuffPos,pSrcDatas,i);
		// On encode
		beEncodeChunk(hbeStrm,iPqSize/2,(PSHORT)pTmpBuff,pMP3Buff,&dwCoded);
		// Et on sauve le rsultat dans le fichier
		fwrite(pMP3Buff,1,dwCoded,f_out);
		// Incrmenter le compteur de taille globale
		dwDataSize+=dwCoded;
		// Passer au paquet suivant en se positionnant au dbut dubuffer temporaire
		iTmpBuffPos=0;
		iDataPos+=i;
		pSrcDatas+=i;
	}
	// S'il reste des donnes  encoder, alors on le fera la prochaine fois
	iTmpBuffPos=DataSize-iDataPos;
	if (iTmpBuffPos>0)
	{
		memcpy(pTmpBuff,pSrcDatas,iTmpBuffPos);
	}
	return 1;
}
int CMP3File::Close()
{
	if (!p_bIsOpened) return 0;
	DWORD dwCoded;
	// S'il reste des donnes  encoder, on le fait maintenant
	if (iTmpBuffPos>0)
	{
		beEncodeChunk(hbeStrm,iTmpBuffPos/2,(PSHORT)pTmpBuff,pMP3Buff,&dwCoded);
		fwrite(pMP3Buff,1,dwCoded,f_out);
		dwDataSize+=dwCoded;
	}
	// D-initialisation du stream MP3
	if (beDeinitStream(hbeStrm,pMP3Buff,&dwCoded)!=BE_ERR_SUCCESSFUL)
	{
		beCloseStream(hbeStrm);
		fclose(f_out);
		MessageBox(GetActiveWindow(),"beDeinitStream failed...","Error",MB_ICONEXCLAMATION);
		return 0;
	}
	if(dwCoded)
	{
		fwrite(pMP3Buff,1,dwCoded,f_out);
		dwDataSize+=dwCoded;
	}
	// Fermeture du Stream
	beCloseStream(hbeStrm);
	// Destruction des buffers
	delete[] pMP3Buff;
	pMP3Buff = NULL;
	int i=sizeof(pTmpBuff);
	delete[] pTmpBuff;
	pTmpBuff = NULL;
	// Fermeture du fichier
	fclose(f_out);
	// Ecriture des TAGs
	beWriteInfoTag(hbeStrm,p_szFName);
	p_bIsOpened=false;
	return 1;
}

void CMP3File::SetEncodingBitRate(DWORD dwBitrate)
{
	p_dwBitrate=dwBitrate;
}

void CMP3File::SetSourceFormat(WORD wChannels,DWORD dwSmplRate,WORD wBits)
{
	p_dwSampleRate=dwSmplRate;
	p_nMode=(wChannels==2)?BE_MP3_MODE_STEREO:BE_MP3_MODE_MONO;
	p_wBits=wBits;
}
int CMP3File::EnableVBR(bool bEnable)
{
	if (p_bIsOpened) return 0;
	p_bEnableVBR=bEnable;
	return 1;
}
int CMP3File::SetVBRMaxBitrate(DWORD dwMax)
{
	if (p_bIsOpened) return 0;
	p_dwMaxBitrate=dwMax;
	return 1;
}
